from osbot_aws.apis.IAM import IAM


class IAM_Policy:
    def __init__(self, policy_name=None, policy_path=None, account_id=None):
        self.iam         = IAM()
        self.policy_name = policy_name
        self.version     = "2012-10-17"
        self.statements  = []
        self.policy_path = policy_path
        self.account_id  = account_id

    def add_cloud_watch(self, resource_arn):
        return self.add_statement_allow(["logs:CreateLogGroup","logs:CreateLogStream","logs:PutLogEvents"], [resource_arn])

    def add_statement(self, effect, actions, resources):
        self.statements.append({"Effect"   : effect    ,
                                "Action"   : actions   ,
                                "Resource" : resources})
        return self

    def add_statement_allow(self, actions, resources):
        return self.add_statement('Allow', actions,resources)

    def create(self):
        if self.policy_name is None:
            return {'status':'error', 'data':'policy name is None'}
        return self.iam.policy_create(self.policy_name, self.statement())

    def delete(self):
        return self.iam.policy_delete(self.policy_arn())

    def exists(self):
        return self.iam.policy_exists(self.policy_arn())

    def policy_arn(self):
        return self.iam.policy_arn(self.policy_name, self.policy_path, self.account_id)

    def statement(self):
        return { 'Version' : self.version , 'Statement': self.statements}

    def statement_from_aws(self):
        return self.iam.policy_statement(self.policy_arn())




import boto3


class Ec2:
    def __init__(self):
        self.ec2          = boto3.client('ec2')
        self.ec2_resource = boto3.resource('ec2')



    def instances_details(self):
        instances = {}
        for instance in self.ec2_resource.instances.all():
            instance_id = instance.instance_id
            instances[instance_id] = { 'cpus'         : instance.cpu_options['CoreCount']     ,
                                       'image_id'     : instance.image_id                     ,
                                       'instance_type': instance.instance_type                ,
                                       'public_ip'    : instance.public_ip_address            ,
                                       'state'        : instance.state                        ,
                                       'tags'         : instance.tags                         }

        return instances

    def security_groups(self):
        groups={}
        for group in self.ec2.describe_security_groups().get('SecurityGroups'):
            groups[group.get('GroupId')] = group
        return groups

    def subnets(self):
        subnets = {}
        for subnet in self.ec2.describe_subnets().get('Subnets'):
            subnets[subnet.get('SubnetId')] = subnet
        return subnets

    def vpcs(self):
        vpcs = {}
        for vpc in self.ec2.describe_vpcs().get('Vpcs'):
            vpcs[vpc.get('VpcId')] = vpc
        return vpcs


import boto3
from   boto3    import resource

class Dynamo:
    def __init__(self):
        self.resource   = resource('dynamodb')
        self.client     = boto3.client('dynamodb')

    def create(self, table_name, key):
        keySchema             = [ {'AttributeName'    : key        , 'KeyType'           : 'HASH' } ]
        attributeDefinitions  = [ {'AttributeName'    : key        , 'AttributeType'     : 'S'    } ]
        provisionedThroughput = { 'ReadCapacityUnits' : 5          , 'WriteCapacityUnits': 5      }
        self.client.create_table( TableName             = table_name           ,
                                  KeySchema             = keySchema            ,
                                  AttributeDefinitions  = attributeDefinitions ,
                                  ProvisionedThroughput = provisionedThroughput)
        self.client.get_waiter('table_exists') \
            .wait(TableName=table_name, WaiterConfig={'Delay': 2, 'MaxAttempts': 10})

    def delete(self, table_name):
        self.client.delete_table(TableName = table_name)
        self.client.get_waiter('table_not_exists')      \
                   .wait(TableName=table_name, WaiterConfig={'Delay': 2, 'MaxAttempts':10 })

    def list(self):
        return self.client.list_tables()['TableNames']

class Dynamo_Table:
    def __init__(self,table_name, key):
        self.table_name = table_name
        self.key        = key
        self.dynamo     = Dynamo()
        self.client     = self.dynamo.client
        self.resource   = self.dynamo.resource
        self.table      = self.resource.Table(self.table_name)
        self.chuck_size = 100

        self._keys      = None

    def add(self,row):
        self.table.put_item(Item=row)
        return row

    def add_batch(self, items):
        with self.table.batch_writer() as batch:
            for item in items:
                batch.put_item(Item=item)

    def delete(self, item):
        self.table.delete_item(Key= { self.key : item })

    def delete_batch(self, items):
        with self.table.batch_writer() as batch:
            for item in items:
                batch.delete_item(Key= item )

    def exists(self):
        if self.info():
            return True
        return False

    def get(self, key_value):
        data = self.table.get_item(Key={self.key: key_value})
        return data.get('Item')

    def get_batch(self, keys):
        def chunks(items,split):
            for i in range(0, len(items), split):
                yield items[i:i + split]

        for chunck in chunks(keys, self.chuck_size):
            request_items = []
            for key in chunck:
                request_items.append({self.key:  key })
            response = self.resource.batch_get_item(RequestItems={self.table_name: {'Keys': request_items}})
            yield response['Responses'][self.table_name]

    def info(self):
        try:
            return self.client.describe_table(TableName = self.table_name)
        except:
            return None

    def keys(self):
        if self._keys:
            return self._keys
        def get_scan_batch (start_key):
            if start_key:
                return self.table.scan(ProjectionExpression=self.key, ExclusiveStartKey =start_key );
            else:
                return self.table.scan(ProjectionExpression=self.key)

        keys = []

        def map_Items(items):
            for item in items:
                 keys.append(item[self.key])
            #print(len(items))

        response = get_scan_batch(None)                                 # first call
        map_Items(response['Items'])                                    # map items
        while 'LastEvaluatedKey' in response:                           # see if we need to page
            response = get_scan_batch(response.get('LastEvaluatedKey')) # call with LastEvaluatedKey
            map_Items(response['Items'])                                # map items

        self._keys = keys

        return keys

    def status(self):

        return self.info()['Table']['TableStatus']


import os
import boto3
import json
import shutil

class Aws_Cli:
    def __init__(self, **kwargs):
        self.aws_lambda = boto3.client('lambda', region_name = kwargs.get('region_name'))
        self.aws_s3     = boto3.client('s3'    , region_name = kwargs.get('region_name'))


        #

    # def lambda_delete_function      (self, name                                                                 ):
    #     if self.lambda_function_exists(name):
    #         self.aws_lambda.delete_function( FunctionName = name)
    #     return self
    #     #

    # def lambda_functions            (self                                                                       ):
    #     data = {}
    #     for function in self.aws_lambda.list_functions().get('Functions'):
    #         data[function['FunctionName']] = function
    #     return data



        #

    # def lambda_function_info        (self, name                                                                 ):
    #     return self.aws_lambda.get_function(FunctionName = name)

    def lambda_invoke_function      (self, name, payload                                                        ):
        response      = self.aws_lambda.invoke(FunctionName=name, Payload = json.dumps(payload) )

        result_bytes  = response.get('Payload').read()
        result_string = result_bytes.decode('utf-8')
        result        = json.loads(result_string)
        return result, response

        #

    def lambda_invoke_function_async(self, name, payload                                                        ):
        return  self.aws_lambda.invoke(FunctionName=name, Payload = json.dumps(payload), InvocationType='Event')

    def lambda_update_function      (self, name, s3_bucket,s3_key                                               ):
        return self.aws_lambda.update_function_code(FunctionName = name,
                                                    S3Bucket     = s3_bucket ,
                                                    S3Key        = s3_key    )

        #

    def s3_buckets                  (self                                                                       ):
        data = {}
        for bucket in self.aws_s3.list_buckets().get('Buckets'):
            data[bucket['Name']] = bucket
        return data

        #

    #def s3_add_notification        (self, s)



import datetime

import boto3

from pbx_gs_python_utils.utils.Dev import Dev

from osbot_aws.apis.Boto_Helpers import Boto_Helpers


class Cloud_Trail(Boto_Helpers):

    def __init__(self):
        self.cloudtrail = boto3.client('cloudtrail')

    def events(self):
        endtime = datetime.datetime.utcnow()
        starttime = datetime.datetime.utcnow() - datetime.timedelta(minutes=55)

        return Boto_Helpers.invoke_using_paginator(self.cloudtrail,'lookup_events','Events',
                                                    LookupAttributes=[                ],
                                                    StartTime=starttime,
                                                    EndTime=endtime, MaxResults=1000)


import os

import pbx_gs_python_utils
from pbx_gs_python_utils.utils.Dev import Dev
from pbx_gs_python_utils.utils.Files import Files

from osbot_aws._tmp_utils.Temp_Files import Temp_Files
from osbot_aws.apis.Lambda import Lambda
from osbot_aws.apis.test_helpers.Temp_Aws_Roles import Temp_Aws_Roles
from osbot_aws.apis.test_helpers.Temp_Lambda import Temp_Folder_Code


class Lambda_Package:
    def __init__(self,lambda_name):
        self.lambda_name   = lambda_name
        self._lambda       = Lambda(self.lambda_name)
        self.tmp_s3_bucket = 'gs-lambda-tests'
        self.tmp_s3_key    = 'unit_tests/lambdas/{0}.zip'.format(self.lambda_name)
        self.role_arn      = Temp_Aws_Roles().for_lambda_invocation()
        self.tmp_folder    = Files.temp_folder('tmp_lambda_')

        (self._lambda.set_s3_bucket  (self.tmp_s3_bucket)
                     .set_s3_key     (self.tmp_s3_key   )
                     .set_role       (self.role_arn     )
                     .set_folder_code(self.tmp_folder  ))

    # helper methods
    @staticmethod
    def get_root_folder():
        return Files.path_combine(__file__, '../..')

    # Lambda class wrappers

    def create(self             ): return self._lambda.create()
    def delete(self             ): return self._lambda.delete()
    def invoke(self, params=None): return self._lambda.invoke(params)
    def update(self             ): return self._lambda.update()

    # main methods

    def add_folder(self, source):
        destination = Files.path_combine(self.tmp_folder,Files.file_name(source))
        Temp_Files.folder_copy(source, destination)
        self.remove_files('__pycache__')
        return self

    def add_root_folder(self):
        self.add_folder(self.get_root_folder())
        return self

    def add_pbx_gs_python_utils(self):
        lib_path = Files.folder_name(pbx_gs_python_utils.__file__)
        self.add_folder(lib_path)
        return self

    def get_files(self):
        all_files = []
        for root, dirs, files in os.walk(self.tmp_folder):
            for file in files:
                file_path = Files.path_combine(root,file).replace(self.tmp_folder,'')
                all_files.append(file_path)
        return all_files

    def remove_files(self,pattern):
        for file in self.get_files():
            if pattern in file:
                file_to_delete = Files.path_combine(self.tmp_folder,file[1:])
                Files.delete(file_to_delete)

    def use_lambda_file(self,lambda_file):
        file_path = Files.path_combine(self.get_root_folder(), lambda_file)
        if Files.exists(file_path) is False:
            return { 'status': 'error', 'data': 'could not find lambda file `{0}` in root folder `{1}`'.format(lambda_file, self.get_root_folder())}
        target_file = Files.path_combine(self.tmp_folder, '{0}.py'.format(self.lambda_name))
        Files.copy(file_path,target_file)
        return  { 'status': 'ok', 'file_path': file_path, 'target_file': target_file }


from pbx_gs_python_utils.utils.Dev import Dev

from osbot_aws.apis.IAM import IAM
from osbot_aws.helpers.IAM_Policy import IAM_Policy


class IAM_Role:
    def __init__(self,role_name):
        self.role_name  = role_name
        self.iam        = IAM(role_name=self.role_name)
        self.role_arn   = None
        self.policy_arn = None

    def add_policy_for__lambda(self):
        temp_policy_name = 'policy_{0}'.format(self.role_name)
        cloud_watch_arn  = "arn:aws:logs:{0}:{1}:log-group:/aws/lambda/*".format('eu-west-2','244560807427')
        iam_policy       = IAM_Policy(temp_policy_name)
        self.policy_arn  = iam_policy.add_cloud_watch(cloud_watch_arn).create().get('policy_arn')
        self.iam.role_policy_attach(self.policy_arn)
        return self


    def create_for__lambda(self):
        result = self.create_for_service('lambda.amazonaws.com')
        if result.get('status') == 'ok':
            self.add_policy_for__lambda()
        return result

    def create_for__code_build(self):
        return self.create_for_service('codebuild.amazonaws.com')

    def create_for_service(self,service):
        role_arn =  self.iam.role_arn()
        if role_arn:
            return {'status':'warning', 'data': 'role already exists', 'role_name': self.iam.role_name , 'role_arn': role_arn}
        else:
            policy_document = {'Statement': [{'Action': 'sts:AssumeRole',
                                              'Effect': 'Allow',
                                              'Principal': {'Service': service}}]}
            data = self.iam.role_create(policy_document)
            return {'status': 'ok', 'data': data, 'role_name': self.iam.role_name, 'role_arn': data.get('Arn') }



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    version                       = "0.1.01"               , # change this on every release
    name                          = "osbot_aws"  ,

    author                        = "Dinis Cruz",
    author_email                  = "dinis.cruz@owasp.org",
    description                   = "OWASP Security Bot - AWS",
    long_description              = long_description,
    long_description_content_type = " text/markdown",
    url                           = "https://github.com/pbx-gs/OSBot-AWS",
    packages                      = setuptools.find_packages(),
    classifiers                   = [ "Programming Language :: Python :: 3"   ,
                                      "License :: OSI Approved :: MIT License",
                                      "Operating System :: OS Independent"   ])